#!/usr/bin/perl -Tw
#
# LMS version 1.8.1 Shyla _ moded by ReYu
#
#  (C) 2001-2011 LMS Developers & Simson
#
#  Please, see the doc/AUTHORS for more information about authors!
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License Version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
#  USA.
#

use strict;
use DBI;
use Config::IniFiles;
use Getopt::Long;
use HTTP::Cookies;
use HTTP::Request::Common qw(POST GET);
use Crypt::SSLeay;
use LWP::UserAgent;
use POSIX;
use POSIX qw(mktime strftime);
use HTML::TokeParser;
use Time::Local;
use vars qw($configfile $quiet $help $version);
use Digest::MD5;
use Text::Iconv;
use Digest::MD5 qw(md5_hex);

my $_version = '1.8.2 Shyla moded by ReYu & Simson';

my %options = (
				"--config-file|C=s"     =>      \$configfile,
				"--quiet|q"             =>      \$quiet,
				"--help|h"              =>      \$help,
				"--version|v"           =>      \$version,
			  );

Getopt::Long::config("no_ignore_case");
GetOptions(%options);

if($help)
{
		print STDERR <<EOF;
lms-cashimport-inteligo, version $_version
		(C) 2001-2011 LMS Developers & ReYu & Simson

		-C, --config-file=/etc/lms/lms.ini      alternate config file (default: /etc/lms/lms.ini);
		-h, --help                      print this help and exit;
		-v, --version                   print version info and exit;
		-q, --quiet                     suppress any output, except errors;

EOF
	exit 0;
}

if($version)
{
		print STDERR <<EOF;
lms-cashimport-inteligo, version $_version
		(C) 2001-2011 LMS Developers & ReYu & Simson

EOF
	exit 0;
}

if(!$configfile)
{
		$configfile = "/etc/lms/lms.ini";
}

if(!$quiet)
{
		print STDOUT "lms-cashimport-inteligo, version $_version\n";
		print STDOUT "(C) 2001-2011 LMS Developers & ReYu & Simson\n";
		print STDOUT "Using file $configfile as config.\n";
}

if(! -r $configfile)
{
		print STDERR "Fatal error: Unable to read configuration file $configfile, exiting.\n";
		exit 1;
}

my $ini = new Config::IniFiles -file => $configfile;
print @Config::IniFiles::errors;

my $dbtype = $ini->val('database', 'type') || 'mysql';
my $dbhost = $ini->val('database', 'host') || 'localhost';
my $dbuser = $ini->val('database', 'user') || 'root';
my $dbpasswd = $ini->val('database', 'password') || '';
my $dbname = $ini->val('database', 'database') || 'lms';

my $login_i = $ini->val('cashimport_inteligo', 'id') || '12345678';
my $passwd_i = $ini->val('cashimport_inteligo', 'pass') ||  'abcdef';
my $account_id = $ini->val('cashimport_inteligo', 'account') || '50102055581111111111111111';
my $time_start = $ini->val('cashimport_inteligo', 'statement_start') || strftime "%Y-%m-%d", localtime(time-86400);
my $time_end = $ini->val('cashimport_inteligo', 'statement_end') || strftime "%Y-%m-%d", localtime(time);
my $file_out_dir = $ini->val('cashimport_inteligo', 'history_dir') || '/www/1';


my $dbase;

if($dbtype eq "mysql")
{
		$dbase = DBI->connect("DBI:mysql:database=$dbname;host=$dbhost","$dbuser","$dbpasswd", { RaiseError => 1 });
}
elsif($dbtype eq "postgres")
{
		$dbase = DBI->connect("DBI:Pg:dbname=$dbname;host=$dbhost","$dbuser","$dbpasswd", { RaiseError => 1 });
}
else
{
		print STDERR "Fatal error: unsupported database type: $dbtype, exiting.\n";
		exit 1;
}

# Definicja Agenta http
my $sd ='';
my $cookie_jar = HTTP::Cookies->new;
my $ua = new LWP::UserAgent;
$ua->timeout(30);
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.7.8) Gecko/20050511 Firefox/1.0.4');
$ua->env_proxy();

# Wchodzimy na strone
my $get = new HTTP::Request(GET => 'https://inteligo.pl/secure');
my $res = $ua->request($get);
if ($res->content =~ /Przepraszamy/ ){
		print "ERROR: System chwilowo niedostepny", "\n";
		exit 1;
}

if ($res->content =~ /Identyfikator/ ){
		my $p = HTML::TokeParser->new(\$res->content);
		while ( my $token1 = $p->get_tag("input")){
				if (my $ur = $token1->[1]{name} eq "sd"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $sd = $url2; }
		}
}

print "Logujemy do Serwisu ... \n";
my $post = POST 'https://inteligo.pl/secure', [
sd => $sd,
   form_name => "login",
   btn_ok => "",
   menu => "",
   cif => "$login_i",
   password => "$passwd_i",
   btn_ok => "",
   ];
my $res1 = $ua->request($post);

my $ias_sid = '';
my $back_reload_protection = '';
my $ias_form_protect = '';
my $ias_navi_map = '';

if ($res1->content =~ /Stan Twoich/ ){
		print "Jestesmy zalogowani ...\n";
		my $p = HTML::TokeParser->new(\$res1->content);
		while ( my $token1 = $p->get_tag("input")){
				if (my $ur = $token1->[1]{name} eq "ias_sid"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_sid = $url2; }
				if (my $ur = $token1->[1]{name} eq "back_reload_protection"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $back_reload_protection = $url2; }
				if (my $ur = $token1->[1]{name} eq "ias.form_protect"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_form_protect = $url2; }
				if (my $ur = $token1->[1]{name} eq "ias_navi_map"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_navi_map = $url2; }
		}
} else {
		exit 1;
}

print "Przechodzimy do historii ...\n";
my $post1 = POST 'https://inteligo.pl/secure/igo2', [
ias_sid => $ias_sid,
		__ie__bug__workaround__ => "",
		ias_navi_map => $ias_navi_map,
		ias_post_into => "/accounts/accounts/submit",
		back_reload_protection => $back_reload_protection,
		"ias.form_protect" => $ias_form_protect,
		menu => "",
		dynsub_10 => "",
		];
$res = $ua->request($post1);
if ($res->content =~ /Historia rachunku/ ){
		print "Strona Historii Rachunku \n";
		my $p1 = HTML::TokeParser->new(\$res->content);
		while ( my $token1 = $p1->get_tag("input")){
				if (my $ur = $token1->[1]{name} eq "ias_sid"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_sid = $url2; }
				if (my $ur = $token1->[1]{name} eq "back_reload_protection"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $back_reload_protection = $url2; }
				if (my $ur = $token1->[1]{name} eq "ias.form_protect"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_form_protect = $url2; }
				if (my $ur = $token1->[1]{name} eq "ias_navi_map"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_navi_map = $url2; }
		}
} else {
		print "\nExit\n";
		exit 1;
}
print "Zmieniamy filtr historii....\n";
my $post2 = POST 'https://inteligo.pl/secure/igo2', [
ias_sid => $ias_sid,
		__ie__bug__workaround__ => "",
		ias_navi_map => $ias_navi_map,
		back_reload_protection => $back_reload_protection,
		"ias.form_protect" => $ias_form_protect,
		menu => "",
		ias_post_into => "/accounts/history/submit",
		"fields_search-selected_acc" => $account_id,
		"fields_search-date_range-since" => $time_start,
		"fields_search-date_range-to" => $time_end,
		format => "pdf",
		dynsub_11 => "",
		];
$res = $ua->request($post2);
if ($res->content =~ /Historia rachunku/ ){
		print "Zmieniono filtr $time_start - $time_end\n";
		my $p1 = HTML::TokeParser->new(\$res->content);
		while (my $token1 = $p1->get_tag("input")){
				if (my $ur = $token1->[1]{name} eq "ias_sid"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_sid = $url2; }
				if (my $ur = $token1->[1]{name} eq "back_reload_protection"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $back_reload_protection = $url2; }
				if (my $ur = $token1->[1]{name} eq "ias.form_protect"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_form_protect = $url2; }
				if (my $ur = $token1->[1]{name} eq "ias_navi_map"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_navi_map = $url2; }
		}
} else {
		print "\nExit\n";
		exit 1;
}

print "Pobieramy plik historii....\t";
my $post3 = POST 'https://inteligo.pl/secure/igo2', [
ias_sid => $ias_sid,
		__ie__bug__workaround__ => "",
		ias_navi_map => $ias_navi_map,
		back_reload_protection => $back_reload_protection,
		"ias.form_protect" => $ias_form_protect,
		menu => "",
		dynsub_12 => "",
		ias_post_into => "/accounts/history/submit",
		"fields_search-selected_acc" => $account_id,
		"fields_search-date_range-since" => $time_start,
		"fields_search-date_range-to" => $time_end,
		format => "csv",
		];
$res = $ua->request($post3);
print "Pobrano plik\n";
my @dana = split("\n", $res->content);
my $file_name = $file_out_dir."/". $account_id ."_". time();
if ($file_out_dir){
		open (DOUT,">$file_name");
		my $y = scalar(@dana) - 1 ;
		print DOUT "Ilosc pozycji : ". $y ."\n"
}

my $okCounter = 0;


for (my $x=1;$x<scalar(@dana);$x++){
		my $line = $dana[$x];



# Line format:
# 2011-11-18,999,Przelew na rachunek,NAZWISKO IMIE NADAWCY ADRES Nr rach.: 00111122223333444455556666 Tytu: KLIENT 15,"+66,66","4 444,44"
		$line =~ m/(.*),(.*),Przelew na rachunek,"?([^"]*) Nr rach.: ([^"]*) Tytu.: ([^"]*)"?,"(.*)","(.*)"/;
		if (!$1) {
				next;
		}

		my $statementid = "$2";
		my $date = "$1";
		my $value = "$6";
		my $account ="$4";
		my $sender = "$3";
		my $description = "$5";
#		my $customerid = "NULL";
		my $customerid = 0;
		my $customer = "BRAK!";
print "sender $sender\n";
		$value =~ s/,/./g;
		my $fullDescription = "$sender<BR/>\n$description<BR/>\n$account<BR/>\nimport_id: $statementid";

		if ($file_out_dir){
				print DOUT $line ."\n";
		}

#		if ($description =~ m/KLIENT *([0-9]*)/) {
#				$customerid=$1;
#		}


    if($account) {
                my $sdbq = $dbase->prepare("SELECT id, lastname, name, address FROM customers WHERE info LIKE '%$account%'");
                $sdbq->execute;
                if (my $row = $sdbq->fetchrow_hashref()){
                        $customerid = $row->{'id'};
                        $customer = $row->{'lastname'}." ".$row->{'name'}." ".$row->{'address'}." id:".$row->{'id'} ;
                }
        }


            my $hash = md5_hex($date.$value.$statementid.$description);



if($customerid == '0'){

my $converter = Text::Iconv->new("ISO-8859-2","UTF-8");

$sender = $converter->convert($sender);
$description = $converter->convert($description);
$fullDescription = $converter->convert($fullDescription);



my  @da = split(" ", $sender);

if( @da>1){


    my $sdbq = $dbase->prepare("SELECT id FROM customers WHERE UPPER(lastname)=UPPER('$da[0]') and UPPER(name)=UPPER('$da[1]')");
       $sdbq->execute;
    my $id = '0';
    if (my $row = $sdbq->fetchrow_hashref()){
        $customer = $sender." id:".$row->{'id'};
        $customerid = $row->{'id'};
    }else {
        my $sdbq = $dbase->prepare("SELECT id FROM customers WHERE UPPER(lastname)=UPPER('$da[1]') and UPPER(name)=UPPER('$da[0]')");
                $sdbq->execute;
                if (my $row = $sdbq->fetchrow_hashref()){
                        $customer = $sender." id:".$row->{'id'};
                        $customerid = $row->{'id'};
                }
        }
}

}


		my $sdbq = $dbase->prepare("SELECT id FROM cashimport WHERE hash = '$hash'");
		$sdbq->execute();
		my $row = $sdbq->fetchrow_hashref();
		if ($row->{'id'}) {
        print "ERROR: Hash exists, ignoring $customer \n\n";
#				exit 1;
				next;
		}




if($customerid == '0'){
$customerid = 67;
}

		my $sdbq = $dbase->prepare("INSERT INTO cashimport ( Date , Value , Customer , Description , CustomerId , Hash ) VALUES (EXTRACT(EPOCH FROM TIMESTAMP WITH TIME ZONE '$date'), '$value', '$customer', '$fullDescription', $customerid, '$hash');");
		print "INSERT INTO cashimport ( Date , Value , Customer , Description , CustomerId , Hash ) VALUES (UNIX_TIMESTAMP('$date'), '$value', '$customer', '$fullDescription', $customerid, '$hash');\n\n";
		$sdbq->execute();
#       print $date ." ".$value." ".$customerid ." ". $hash ."\n";
$okCounter ++;
}

close DOUT;

print "OK: $okCounter\n";

exit 1;
